@@ -9,6 +9,10 @@
 #include <ctype.h>
 #include <stdlib.h>
 
+#ifdef EDB_NATIVE_LANG
+#include "access/tuptoaster.h"
+#endif
+
 /*
  * get_control_data()
  *
@@ -23,9 +27,7 @@
  * pg_control data.
  */
 void
-get_control_data(migratorContext *ctx, const char *bindir,
-				 const char *datadir, ControlData *ctrl,
-				 const uint32 pg_version)
+get_control_data(migratorContext *ctx, ClusterInfo *cluster)
 {
 	char		cmd[MAXPGPATH];
 	char		bufin[MAX_STRING];
@@ -56,7 +58,8 @@ get_control_data(migratorContext *ctx, const char *bindir,
 #else
 	SetEnvironmentVariableA("LANG", "C");
 #endif
-	sprintf(cmd, SYSTEMQUOTE "\"%s/pg_resetxlog\" -n \"%s\"" SYSTEMQUOTE, bindir, datadir);
+	sprintf(cmd, SYSTEMQUOTE "\"%s/pg_resetxlog\" -n \"%s\"" SYSTEMQUOTE,
+			cluster->bindir, cluster->pgdata);
 	fflush(stdout);
 	fflush(stderr);
 
@@ -65,16 +68,25 @@ get_control_data(migratorContext *ctx, const char *bindir,
 			   strerror(errno));
 
 	/* Only pre-8.4 has these so if they are not set below we will check later */
-	ctrl->lc_collate = NULL;
-	ctrl->lc_ctype = NULL;
+	cluster->controldata.lc_collate = NULL;
+	cluster->controldata.lc_ctype = NULL;
 
 	/* Only in <= 8.3 */
-	if (GET_MAJOR_VERSION(pg_version) <= 803)
+	if (GET_MAJOR_VERSION(cluster->pg_version) <= 803)
 	{
-		ctrl->float8_pass_by_value = false;
+		cluster->controldata.float8_pass_by_value = false;
 		got_float8_pass_by_value = true;
 	}
 
+#ifdef EDB_NATIVE_LANG
+	/* EDB AS 8.3 is an 8.2 code base */
+	if (cluster->is_edb_as && GET_MAJOR_VERSION(cluster->pg_version) <= 803)
+	{
+		cluster->controldata.toast = TOAST_MAX_CHUNK_SIZE;
+		got_toast = true;
+	}
+#endif
+
 	/* we have the result of cmd in "output". so parse it line by line now */
 	while (fgets(bufin, sizeof(bufin), output))
 	{
@@ -106,7 +118,7 @@ get_control_data(migratorContext *ctx, const char *bindir,
 				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
-			ctrl->ctrl_ver = (uint32) atol(p);
+			cluster->controldata.ctrl_ver = (uint32) atol(p);
 		}
 		else if ((p = strstr(bufin, "Catalog version number:")) != NULL)
 		{
@@ -116,9 +128,11 @@ get_control_data(migratorContext *ctx, const char *bindir,
 				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
-			ctrl->cat_ver = (uint32) atol(p);
+			cluster->controldata.cat_ver = (uint32) atol(p);
 		}
-		else if ((p = strstr(bufin, "First log file ID after reset:")) != NULL)
+		else if ((p = strstr(bufin, "First log file ID after reset:")) != NULL ||
+				 (cluster->is_edb_as && GET_MAJOR_VERSION(cluster->pg_version) <= 803 &&
+				  (p = strstr(bufin, "Current log file ID:")) != NULL))
 		{
 			p = strchr(p, ':');
 
@@ -126,10 +140,12 @@ get_control_data(migratorContext *ctx, const char *bindir,
 				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
-			ctrl->logid = (uint32) atol(p);
+			cluster->controldata.logid = (uint32) atol(p);
 			got_log_id = true;
 		}
-		else if ((p = strstr(bufin, "First log file segment after reset:")) != NULL)
+		else if ((p = strstr(bufin, "First log file segment after reset:")) != NULL ||
+				 (cluster->is_edb_as && GET_MAJOR_VERSION(cluster->pg_version) <= 803 &&
+				  (p = strstr(bufin, "Next log file segment:")) != NULL))
 		{
 			p = strchr(p, ':');
 
@@ -137,7 +153,7 @@ get_control_data(migratorContext *ctx, const char *bindir,
 				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
-			ctrl->nxtlogseg = (uint32) atol(p);
+			cluster->controldata.nxtlogseg = (uint32) atol(p);
 			got_log_seg = true;
 		}
 		else if ((p = strstr(bufin, "Latest checkpoint's TimeLineID:")) != NULL)
@@ -148,7 +164,7 @@ get_control_data(migratorContext *ctx, const char *bindir,
 				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
-			ctrl->chkpnt_tli = (uint32) atol(p);
+			cluster->controldata.chkpnt_tli = (uint32) atol(p);
 			got_tli = true;
 		}
 		else if ((p = strstr(bufin, "Latest checkpoint's NextXID:")) != NULL)
@@ -162,7 +178,7 @@ get_control_data(migratorContext *ctx, const char *bindir,
 				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
 
 			op++;				/* removing ':' char */
-			ctrl->chkpnt_nxtxid = (uint32) atol(op);
+			cluster->controldata.chkpnt_nxtxid = (uint32) atol(op);
 			got_xid = true;
 		}
 		else if ((p = strstr(bufin, "Latest checkpoint's NextOID:")) != NULL)
@@ -173,7 +189,7 @@ get_control_data(migratorContext *ctx, const char *bindir,
 				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
-			ctrl->chkpnt_nxtoid = (uint32) atol(p);
+			cluster->controldata.chkpnt_nxtoid = (uint32) atol(p);
 			got_oid = true;
 		}
 		else if ((p = strstr(bufin, "Maximum data alignment:")) != NULL)
@@ -184,7 +200,7 @@ get_control_data(migratorContext *ctx, const char *bindir,
 				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
-			ctrl->align = (uint32) atol(p);
+			cluster->controldata.align = (uint32) atol(p);
 			got_align = true;
 		}
 		else if ((p = strstr(bufin, "Database block size:")) != NULL)
@@ -195,7 +211,7 @@ get_control_data(migratorContext *ctx, const char *bindir,
 				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
-			ctrl->blocksz = (uint32) atol(p);
+			cluster->controldata.blocksz = (uint32) atol(p);
 			got_blocksz = true;
 		}
 		else if ((p = strstr(bufin, "Blocks per segment of large relation:")) != NULL)
@@ -206,7 +222,7 @@ get_control_data(migratorContext *ctx, const char *bindir,
 				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
-			ctrl->largesz = (uint32) atol(p);
+			cluster->controldata.largesz = (uint32) atol(p);
 			got_largesz = true;
 		}
 		else if ((p = strstr(bufin, "WAL block size:")) != NULL)
@@ -217,7 +233,7 @@ get_control_data(migratorContext *ctx, const char *bindir,
 				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
-			ctrl->walsz = (uint32) atol(p);
+			cluster->controldata.walsz = (uint32) atol(p);
 			got_walsz = true;
 		}
 		else if ((p = strstr(bufin, "Bytes per WAL segment:")) != NULL)
@@ -228,7 +244,7 @@ get_control_data(migratorContext *ctx, const char *bindir,
 				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
-			ctrl->walseg = (uint32) atol(p);
+			cluster->controldata.walseg = (uint32) atol(p);
 			got_walseg = true;
 		}
 		else if ((p = strstr(bufin, "Maximum length of identifiers:")) != NULL)
@@ -239,7 +255,7 @@ get_control_data(migratorContext *ctx, const char *bindir,
 				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
-			ctrl->ident = (uint32) atol(p);
+			cluster->controldata.ident = (uint32) atol(p);
 			got_ident = true;
 		}
 		else if ((p = strstr(bufin, "Maximum columns in an index:")) != NULL)
@@ -250,7 +266,7 @@ get_control_data(migratorContext *ctx, const char *bindir,
 				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
-			ctrl->index = (uint32) atol(p);
+			cluster->controldata.index = (uint32) atol(p);
 			got_index = true;
 		}
 		else if ((p = strstr(bufin, "Maximum size of a TOAST chunk:")) != NULL)
@@ -261,7 +277,7 @@ get_control_data(migratorContext *ctx, const char *bindir,
 				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
-			ctrl->toast = (uint32) atol(p);
+			cluster->controldata.toast = (uint32) atol(p);
 			got_toast = true;
 		}
 		else if ((p = strstr(bufin, "Date/time type storage:")) != NULL)
@@ -272,7 +288,7 @@ get_control_data(migratorContext *ctx, const char *bindir,
 				pg_log(ctx, PG_FATAL, "%d: pg_resetxlog  problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
-			ctrl->date_is_int = strstr(p, "64-bit integers") != NULL;
+			cluster->controldata.date_is_int = strstr(p, "64-bit integers") != NULL;
 			got_date_is_int = true;
 		}
 		else if ((p = strstr(bufin, "Float8 argument passing:")) != NULL)
@@ -284,7 +300,7 @@ get_control_data(migratorContext *ctx, const char *bindir,
 
 			p++;				/* removing ':' char */
 			/* used later for /contrib check */
-			ctrl->float8_pass_by_value = strstr(p, "by value") != NULL;
+			cluster->controldata.float8_pass_by_value = strstr(p, "by value") != NULL;
 			got_float8_pass_by_value = true;
 		}
 		/* In pre-8.4 only */
@@ -300,7 +316,7 @@ get_control_data(migratorContext *ctx, const char *bindir,
 			p += strspn(p, " ");
 			if (strlen(p) > 0 && *(p + strlen(p) - 1) == '\n')
 				*(p + strlen(p) - 1) = '\0';
-			ctrl->lc_collate = pg_strdup(ctx, p);
+			cluster->controldata.lc_collate = pg_strdup(ctx, p);
 		}
 		/* In pre-8.4 only */
 		else if ((p = strstr(bufin, "LC_CTYPE:")) != NULL)
@@ -315,7 +331,7 @@ get_control_data(migratorContext *ctx, const char *bindir,
 			p += strspn(p, " ");
 			if (strlen(p) > 0 && *(p + strlen(p) - 1) == '\n')
 				*(p + strlen(p) - 1) = '\0';
-			ctrl->lc_ctype = pg_strdup(ctx, p);
+			cluster->controldata.lc_ctype = pg_strdup(ctx, p);
 		}
 	}
 
