@@ -18,6 +18,7 @@
 
 
 static void install_support_functions(migratorContext *ctx);
+static void uninstall_support_functions(migratorContext *ctx);
 static void generate_old_dump(migratorContext *ctx);
 static void split_old_dump(migratorContext *ctx);
 static void copy_clog_xlog_xid(migratorContext *ctx);
@@ -73,6 +74,9 @@ main(int argc, char **argv)
 	{
 		v8_3_check_for_name_data_type_usage(&ctx, CLUSTER_OLD);
 		v8_3_check_for_tsquery_usage(&ctx, CLUSTER_OLD);
+		v8_3_check_for_composite_types(&ctx, CLUSTER_OLD);
+		v8_3_check_for_array_types(&ctx, CLUSTER_OLD);
+		v8_3_check_for_enum_types(&ctx, CLUSTER_OLD);
 		if (ctx.check)
 		{
 			v8_3_rebuild_tsvector_tables(&ctx, true, CLUSTER_OLD);
@@ -118,23 +122,15 @@ main(int argc, char **argv)
 		exit_nicely(&ctx, false);
 	}
 
-	pg_log(&ctx, PG_REPORT, "*Checks complete*\n\n"
-		"| If pg_migrator fails after this point, you must\n"
-		"| re-initdb the new cluster before continuing.\n");
+	pg_log(&ctx, PG_REPORT, "*Checks complete*\n\n");
 
-	if (ctx.num_tablespaces)
-		pg_log(&ctx, PG_REPORT,
-			"| You will also need to remove the \".old\" suffix\n"
-			"| from old $PGDATA/global/pg_control and old\n"
-			"| tablespace directory names before continuing.\n");
-		
-	pg_log(&ctx, PG_REPORT, "\nPerforming migration\n");
-	pg_log(&ctx, PG_REPORT, "--------------------\n");
+	pg_log(&ctx, PG_REPORT, "Preparing for migration\n");
+	pg_log(&ctx, PG_REPORT, "-----------------------\n");
 	
+	/*
+	 *	Non-Destructive Changes
+	 */
 
-	/* rename pg_control so old server cannot be accidentally started */
-	rename_old_pg_control(&ctx);
-	
 	/* -- NEW -- */
 	start_postmaster(&ctx, CLUSTER_NEW, false);
 
@@ -158,9 +154,32 @@ main(int argc, char **argv)
 			  ctx.new.bindir, ctx.new.port, ctx.logfile);
 	check_ok(&ctx);
 
+	get_misc_relfilenodes(&ctx, CLUSTER_NEW);
+
 	stop_postmaster(&ctx, false, false);
 
 
+	pg_log(&ctx, PG_REPORT, "\n"
+		"| If pg_migrator fails after this point, you must\n"
+		"| re-initdb the new cluster before continuing.\n");
+
+	if (ctx.num_tablespaces)
+		pg_log(&ctx, PG_REPORT,
+			"| You will also need to remove the \".old\" suffix\n"
+			"| from old $PGDATA/global/pg_control and old\n"
+			"| tablespace directory names before continuing.\n");
+		
+
+	pg_log(&ctx, PG_REPORT, "\nPerforming migration\n");
+	pg_log(&ctx, PG_REPORT, "--------------------\n");
+
+	/*
+	 *	Destructive Changes
+	 */
+
+	/* rename pg_control so old server cannot be accidentally started */
+	rename_old_pg_control(&ctx);
+	
 	/*
 	 *	Rename all tablespace paths to .old so we can copy them later.
 	 *	DESTRUCTIVE of old cluster
@@ -177,7 +196,6 @@ main(int argc, char **argv)
 	start_postmaster(&ctx, CLUSTER_NEW, false);
 
 	set_frozenxids(&ctx);
-	get_misc_relfilenodes(&ctx, CLUSTER_NEW);
 
 	/*
 	 * We have to create the databases first so we can create the
@@ -187,14 +205,12 @@ main(int argc, char **argv)
 	exec_prog(&ctx, true,
 			  SYSTEMQUOTE "\"%s/%s\" --set ON_ERROR_STOP=on --port %d "
 			  "-f \"%s/%s\" --dbname template1 >> \"%s\"" SYSTEMQUOTE,
-			  ctx.new.bindir, ctx.new_psql_exe, ctx.new.port,
+			  ctx.new.bindir, ctx.new.psql_exe, ctx.new.port,
 			  ctx.home_dir, GLOBALS_DUMP_FILE, ctx.logfile);
 	check_ok(&ctx);
 
 	get_db_and_rel_infos(&ctx, &ctx.new.dbarr, CLUSTER_NEW);
 
-	install_support_functions(&ctx);
-
 	stop_postmaster(&ctx, false, false);
 
 	prep_status(&ctx, "Creating placeholder relfiles for toast relations\n");
@@ -210,7 +226,7 @@ main(int argc, char **argv)
 	exec_prog(&ctx, true,
 			  SYSTEMQUOTE "\"%s/%s\" --set ON_ERROR_STOP=on --port %d "
 			  "-f \"%s/%s\" --dbname template1 >> \"%s\"" SYSTEMQUOTE,
-			  ctx.new.bindir, ctx.new_psql_exe, ctx.new.port,
+			  ctx.new.bindir, ctx.new.psql_exe, ctx.new.port,
 			  ctx.home_dir, DB_DUMP_FILE, ctx.logfile);
 	check_ok(&ctx);
 
@@ -218,11 +234,15 @@ main(int argc, char **argv)
 	dbarr_free(&ctx.new.dbarr);
 	get_db_and_rel_infos(&ctx, &ctx.new.dbarr, CLUSTER_NEW);
 
+	install_support_functions(&ctx);
+
 	prep_status(&ctx, "Restoring relations to use fixed toast file names\n");
 	adjust_rels_to_use_fixed_toast_relfilenodes(&ctx, &ctx.old.dbarr, &ctx.new.dbarr);
 	prep_status(&ctx, "");	/* in case nothing printed */
 	check_ok(&ctx);
 
+	uninstall_support_functions(&ctx);
+
 	/*
 	 * adjust_rels_to_use_fixed_toast_relfilenodes would have created new
 	 * entries in the new database; hence regenerate new_dbinfos to get in sync.
@@ -233,7 +253,7 @@ main(int argc, char **argv)
 	stop_postmaster(&ctx, false, false);
 
 
-	prep_status(&ctx, "Restoring user relations\n");
+	prep_status(&ctx, "Restoring user relation files\n");
 	transfer_all_new_dbs(&ctx, &ctx.old.dbarr, &ctx.new.dbarr,
 						 ctx.old.pgdata, ctx.new.pgdata);
 	prep_status(&ctx, "");	/* in case nothing printed */
@@ -254,20 +274,21 @@ main(int argc, char **argv)
 		GET_MAJOR_VERSION(ctx.new.pg_version) >= 804)
 	{
 		start_postmaster(&ctx, CLUSTER_NEW, true);
+
 		/* restore proper sequence values using file created from old server */
-		if (strlen(sequence_script_file_name) > 0)
+		if (sequence_script_file_name)
 		{
 			prep_status(&ctx, "Adjusting sequences");
 			exec_prog(&ctx, true,
 					  SYSTEMQUOTE "\"%s/%s\" --set ON_ERROR_STOP=on --port %d "
 					  "-f \"%s\" --dbname template1 >> \"%s\"" SYSTEMQUOTE,
-					  ctx.new.bindir, ctx.new_psql_exe, ctx.new.port,
+					  ctx.new.bindir, ctx.new.psql_exe, ctx.new.port,
 					  sequence_script_file_name, ctx.logfile);
 			unlink(sequence_script_file_name);
+			pg_free(sequence_script_file_name);
 			check_ok(&ctx);
 		}
-		pg_free(sequence_script_file_name);
-
+	
 		v8_3_rebuild_tsvector_tables(&ctx, false, CLUSTER_NEW);
 		v8_3_invalidate_hash_gin_indexes(&ctx, false, CLUSTER_NEW);
 		v8_3_invalidate_bpchar_pattern_ops_indexes(&ctx, false, CLUSTER_NEW);
@@ -550,9 +571,17 @@ check_cluster_versions(migratorContext *ctx)
 	if (GET_MAJOR_VERSION(ctx->old.pg_version) < 803)
 		pg_log(ctx, PG_FATAL, "This utility can only upgrade from PostgreSQL version 8.3 and later.\n");
 
-	/* Only PG 8.4+ has enhancements to pg_dump, pg_dumpall, and vacuumdb */
-	if (GET_MAJOR_VERSION(ctx->new.pg_version) < 804)
-		pg_log(ctx, PG_FATAL, "This utility can only upgrade to PostgreSQL version 8.4 and later.\n");
+	if (!ctx->new.is_edb_as)
+	{
+		/* Only PG 8.4+ has enhancements to pg_dump, pg_dumpall, and vacuumdb */
+		if (GET_MAJOR_VERSION(ctx->new.pg_version) < 804)
+			pg_log(ctx, PG_FATAL, "This utility can only upgrade to PostgreSQL version 8.4 and later.\n");
+	}
+	else
+	{
+		if (GET_MAJOR_VERSION(ctx->new.pg_version) < 803)
+			pg_log(ctx, PG_FATAL, "This utility can only upgrade to EnterpriseDB Advanced Server version 8.3 and later.\n");
+	}
 
 	/*
 	 * We can't allow downgrading because we use the target pg_dumpall, and
@@ -567,7 +596,12 @@ check_cluster_versions(migratorContext *ctx)
 		pg_log(ctx, PG_FATAL,
 				"This binary was built using %s PostgreSQL sources; it must be\n"
 				"built using %s PostgreSQL sources to match the new cluster.\n",
-				PG_MAJORVERSION, ctx->new.pg_version_str);
+#if defined(PG_MAJORVERSION) || !defined(EDB_NATIVE_LANG)
+				PG_MAJORVERSION,
+#else
+				"8.3",	/* assume EDB AS 8.3 */
+#endif
+				ctx->new.pg_version_str);
 }
 
 
@@ -594,10 +628,8 @@ check_cluster_compatibility(migratorContext *ctx)
 		fclose(lib_test);
 
 	/* get/check pg_control data of servers */
-	get_control_data(ctx, ctx->old.bindir, ctx->old.pgdata,
-					 &ctx->old.controldata, ctx->old.pg_version);
-	get_control_data(ctx, ctx->new.bindir, ctx->new.pgdata,
-					 &ctx->new.controldata, ctx->new.pg_version);
+	get_control_data(ctx, &ctx->old);
+	get_control_data(ctx, &ctx->new);
 	check_control_data(ctx, &ctx->old.controldata, &ctx->new.controldata);
 }
 
@@ -665,12 +697,12 @@ install_support_functions(migratorContext *ctx)
 		PGconn	   *conn = connectToServer(ctx, newdb->db_name, CLUSTER_NEW);
 
 		PQclear(executeQueryOrDie(ctx, conn,
-					"CREATE FUNCTION public.pg_toasttbl_recreate(OID,OID) "
+					"CREATE OR REPLACE FUNCTION public.pg_toasttbl_recreate(OID,OID) "
 					"RETURNS VOID "
 					"AS '$libdir/pg_migrator' "
 					"LANGUAGE C STRICT;"));
 		PQclear(executeQueryOrDie(ctx, conn,
-					"CREATE FUNCTION public.pg_toasttbl_drop(OID) "
+					"CREATE OR REPLACE FUNCTION public.pg_toasttbl_drop(OID) "
 					"RETURNS VOID "
 					"AS '$libdir/pg_migrator' "
 					"LANGUAGE C STRICT;"));
@@ -681,6 +713,34 @@ install_support_functions(migratorContext *ctx)
 
 
 
+/*
+ * uninstall_support_functions()
+ *
+ * remove pg_migrator support functions
+ */
+static void
+uninstall_support_functions(migratorContext *ctx)
+{
+	int			dbnum;
+
+	prep_status(ctx, "Removing support functions from new cluster");
+
+	for (dbnum = 0; dbnum < ctx->new.dbarr.ndbs; dbnum++)
+	{
+		DbInfo	   *newdb = &ctx->new.dbarr.dbs[dbnum];
+		PGconn	   *conn = connectToServer(ctx, newdb->db_name, CLUSTER_NEW);
+
+		PQclear(executeQueryOrDie(ctx, conn,
+					"DROP FUNCTION public.pg_toasttbl_recreate(OID,OID);"));
+		PQclear(executeQueryOrDie(ctx, conn,
+					"DROP FUNCTION public.pg_toasttbl_drop(OID);"));
+		PQfinish(conn);
+	}
+	check_ok(ctx);
+}
+
+
+
 static void
 rename_old_pg_control(migratorContext *ctx)
 {
