@@ -2,7 +2,7 @@
  *	pg_migrator.h
  */
 
-#define	PG_MIGRATOR_VERSION	"8.4.1"
+#define	PG_MIGRATOR_VERSION	"8.4.2"
 
 #include "postgres.h"
 
@@ -69,7 +69,12 @@ extern int  pgunlink(const char *path);
 
 /* from pgport */
 extern void copydir(char *fromdir, char *todir, bool recurse);
+#if !defined(EDB_NATIVE_LANG) || GET_MAJOR_VERSION(PG_VERSION_NUM) >= 804
 extern bool rmtree(const char *path, bool rmtopdir);
+#else
+/* Used for EDB AS 8.3 compiles */
+extern bool rmtree(char *path, bool rmtopdir);
+#endif
 
 extern char pathSeparator;
 
@@ -197,6 +202,8 @@ typedef struct
 	DbInfoArr	dbarr;		/* dbinfos array */
 	char	   *pgdata;		/* pathname for cluster's $PGDATA directory */
 	char	   *bindir;		/* pathname for cluster's executable directory */
+	const char *psql_exe;	/* name of the psql command to execute
+							 * in the cluster */
 	unsigned short port;	/* port number where postmaster is waiting */
 	uint32		pg_version;		/* PG_VERSION of cluster */
 	char	   *pg_version_str;	/* string PG_VERSION of cluster */
@@ -204,6 +211,8 @@ typedef struct
 	Oid			pg_largeobject_oid;	/* OID of pg_largeobject relation */
 	Oid			pg_largeobject_index_oid;	/* OID of pg_largeobject index */
 	char	   *libpath;	/* pathname for cluster's pkglibdir */
+	/* EDB AS is PG 8.2 with 8.3 enhancements backpatched. */
+	bool		is_edb_as;	/* is EnterpriseDB Advanced Server? */
 } ClusterInfo;
 
 
@@ -225,8 +234,6 @@ typedef struct
 	int			num_tablespaces;
 	char	  **libraries;		/* loadable libraries */
 	int			num_libraries;
-	const char *new_psql_exe;	/* name of the psql command to execute
-								 * in the new cluster */
 	pgpid_t		postmasterPID;	/* PID of currently running postmaster */
 	Cluster	running_cluster;
 
@@ -248,9 +255,7 @@ char		scandir_file_pattern[MAX_STRING + 1];
 
 /* controldata.c */
 
-void get_control_data(migratorContext *ctx, const char *bindir,
-				 const char *datadir, ControlData *ctrl,
-				 const uint32 pg_version);
+void get_control_data(migratorContext *ctx, ClusterInfo *cluster);
 void check_control_data(migratorContext *ctx, ControlData *oldctrl,
 				   ControlData *newctrl);
 void set_locale_and_encoding(migratorContext *ctx, Cluster whichCluster);
@@ -384,6 +389,12 @@ void		v8_3_check_for_name_data_type_usage(migratorContext *ctx,
 							Cluster whichCluster);
 void		v8_3_check_for_tsquery_usage(migratorContext *ctx,
 							Cluster whichCluster);
+void		v8_3_check_for_composite_types(migratorContext *ctx,
+							Cluster whichCluster);
+void		v8_3_check_for_array_types(migratorContext *ctx,
+							Cluster whichCluster);
+void		v8_3_check_for_enum_types(migratorContext *ctx,
+							Cluster whichCluster);
 void		v8_3_check_for_isn_and_int8_passing_mismatch(migratorContext *ctx,
 							Cluster whichCluster);
 void		v8_3_rebuild_tsvector_tables(migratorContext *ctx,
