@@ -6,6 +6,10 @@
 
 #include "pg_migrator.h"
 
+#ifdef EDB_NATIVE_LANG
+#include <fcntl.h>
+#endif
+
 #include <sys/stat.h>
 
 #include "catalog/pg_class.h"
@@ -228,6 +232,8 @@ get_misc_relfilenodes(migratorContext *ctx, Cluster whichCluster)
 	PGresult   *res;
 	int			i_relfile;
 
+	prep_status(ctx, "Getting pg_database and pg_largeobject relfilenodes");
+
 	res = executeQueryOrDie(ctx, conn,
 							"SELECT c.relname, c.relfilenode "
 							"FROM 	pg_catalog.pg_class c, "
@@ -255,6 +261,8 @@ get_misc_relfilenodes(migratorContext *ctx, Cluster whichCluster)
 
 	PQclear(res);
 	PQfinish(conn);
+
+	check_ok(ctx);
 }
 
 
