@@ -187,6 +187,280 @@ v8_3_check_for_tsquery_usage(migratorContext *ctx, Cluster whichCluster)
 }
 
 
+/*
+ * v8_3_check_for_composite_types()
+ *
+ *	composite types have pg_type oids in their data values and
+ *  pg_type.oid is not preserved between migrations.  We don't
+ *	have to worry about arrays of composite types because we
+ *	check arrays later
+ */
+void
+v8_3_check_for_composite_types(migratorContext *ctx, Cluster whichCluster)
+{
+	ClusterInfo	*active_cluster = (whichCluster == CLUSTER_OLD) ?
+					&ctx->old : &ctx->new;
+	int			dbnum;
+	FILE		*script = NULL;
+	bool		found = false;
+	char		output_path[MAXPGPATH];
+
+	prep_status(ctx, "Checking for user columns of composite types");
+
+	snprintf(output_path, sizeof(output_path), "%s/tables_using_composite_types.txt",
+			ctx->home_dir);
+	
+	for (dbnum = 0; dbnum < active_cluster->dbarr.ndbs; dbnum++)
+	{
+		PGresult   *res;
+ 		bool		db_used = false;
+		int			ntups;
+		int			rowno;
+		int			i_nspname, i_relname, i_attname;
+		DbInfo	   *active_db = &active_cluster->dbarr.dbs[dbnum];
+		PGconn	   *conn = connectToServer(ctx, active_db->db_name, whichCluster);
+		
+		/* Find any user-defined tsquery columns */
+		res = executeQueryOrDie(ctx, conn,
+								"SELECT n.nspname, c.relname, a.attname "
+								"FROM	pg_catalog.pg_class c, "
+								"		pg_catalog.pg_namespace n, "
+								"		pg_catalog.pg_attribute a, "
+								"		pg_catalog.pg_type t "
+								"WHERE	c.relkind = 'r' AND "
+								"		c.oid = a.attrelid AND "
+								"		NOT a.attisdropped AND "
+								"		a.atttypid = t.oid AND "
+								"		c.relnamespace = n.oid AND "
+								"		t.typtype = 'c' AND "
+								"		n.nspname != 'pg_catalog' AND "
+								"		n.nspname != 'information_schema'");
+
+		ntups = PQntuples(res);
+		i_nspname = PQfnumber(res, "nspname");
+		i_relname = PQfnumber(res, "relname");
+		i_attname = PQfnumber(res, "attname");
+		for (rowno = 0; rowno < ntups; rowno++)
+		{
+			found = true;
+			if (script == NULL && (script = fopen(output_path, "w")) == NULL)
+					pg_log(ctx, PG_FATAL, "Could not create necessary file:  %s\n", output_path);
+			if (!db_used)
+			{
+				fprintf(script, "Database:  %s\n", active_db->db_name);
+				db_used = true;
+			}
+			fprintf(script, "  %s.%s.%s\n",
+					PQgetvalue(res, rowno, i_nspname),
+					PQgetvalue(res, rowno, i_relname),
+					PQgetvalue(res, rowno, i_attname));
+		}
+
+		PQclear(res);
+
+		PQfinish(conn);
+	}
+
+	if (found)
+	{
+		fclose(script);
+		pg_log(ctx, PG_REPORT, "fatal\n");
+		pg_log(ctx, PG_FATAL,
+				"| Your installation uses composite types.\n"
+				"| These types are not supported for upgrade because\n"
+				"| they contain an internal pg_type.oid that cannot be\n"
+				"| migrated.  You can remove the problem columns and\n"
+				"| restart the migration.  A list of the problem columns\n"
+				"| is in the file:\n"
+				"| \t%s\n\n", output_path);
+	}
+	else
+		check_ok(ctx);
+}
+
+
+/*
+ * v8_3_check_for_array_types()
+ *
+ *	array types have pg_type oids in their data values and
+ *  pg_type.oid is not preserved between migrations.  We catch
+ *	arrays of composite types here too.
+ */
+void
+v8_3_check_for_array_types(migratorContext *ctx, Cluster whichCluster)
+{
+	ClusterInfo	*active_cluster = (whichCluster == CLUSTER_OLD) ?
+					&ctx->old : &ctx->new;
+	int			dbnum;
+	FILE		*script = NULL;
+	bool		found = false;
+	char		output_path[MAXPGPATH];
+
+	prep_status(ctx, "Checking for user columns of array types");
+
+	snprintf(output_path, sizeof(output_path), "%s/tables_using_array_types.txt",
+			ctx->home_dir);
+	
+	for (dbnum = 0; dbnum < active_cluster->dbarr.ndbs; dbnum++)
+	{
+		PGresult   *res;
+ 		bool		db_used = false;
+		int			ntups;
+		int			rowno;
+		int			i_nspname, i_relname, i_attname;
+		DbInfo	   *active_db = &active_cluster->dbarr.dbs[dbnum];
+		PGconn	   *conn = connectToServer(ctx, active_db->db_name, whichCluster);
+		
+		/* Find any user-defined tsquery columns */
+		res = executeQueryOrDie(ctx, conn,
+								"SELECT n.nspname, c.relname, a.attname "
+								"FROM	pg_catalog.pg_class c, "
+								"		pg_catalog.pg_namespace n, "
+								"		pg_catalog.pg_attribute a, "
+								"		pg_catalog.pg_type t "
+								"WHERE	c.relkind = 'r' AND "
+								"		c.oid = a.attrelid AND "
+								"		NOT a.attisdropped AND "
+								"		a.atttypid = t.oid AND "
+								"		c.relnamespace = n.oid AND "
+								"		t.typtype = 'b' AND "
+								"		t.typtype = 'A' AND "
+								"		n.nspname != 'pg_catalog' AND "
+								"		n.nspname != 'information_schema'");
+
+		ntups = PQntuples(res);
+		i_nspname = PQfnumber(res, "nspname");
+		i_relname = PQfnumber(res, "relname");
+		i_attname = PQfnumber(res, "attname");
+		for (rowno = 0; rowno < ntups; rowno++)
+		{
+			found = true;
+			if (script == NULL && (script = fopen(output_path, "w")) == NULL)
+					pg_log(ctx, PG_FATAL, "Could not create necessary file:  %s\n", output_path);
+			if (!db_used)
+			{
+				fprintf(script, "Database:  %s\n", active_db->db_name);
+				db_used = true;
+			}
+			fprintf(script, "  %s.%s.%s\n",
+					PQgetvalue(res, rowno, i_nspname),
+					PQgetvalue(res, rowno, i_relname),
+					PQgetvalue(res, rowno, i_attname));
+		}
+
+		PQclear(res);
+
+		PQfinish(conn);
+	}
+
+	if (found)
+	{
+		fclose(script);
+		pg_log(ctx, PG_REPORT, "fatal\n");
+		pg_log(ctx, PG_FATAL,
+				"| Your installation uses array types.\n"
+				"| These types are not supported for upgrade because\n"
+				"| they contain an internal pg_type.oid that cannot be\n"
+				"| migrated.  You can remove the problem columns and\n"
+				"| restart the migration.  A list of the problem columns\n"
+				"| is in the file:\n"
+				"| \t%s\n\n", output_path);
+	}
+	else
+		check_ok(ctx);
+}
+
+
+/*
+ * v8_3_check_for_enum_types()
+ *
+ *	enum types have pg_type oids in their data values and
+ *  pg_type.oid is not preserved between migrations.
+ */
+void
+v8_3_check_for_enum_types(migratorContext *ctx, Cluster whichCluster)
+{
+	ClusterInfo	*active_cluster = (whichCluster == CLUSTER_OLD) ?
+					&ctx->old : &ctx->new;
+	int			dbnum;
+	FILE		*script = NULL;
+	bool		found = false;
+	char		output_path[MAXPGPATH];
+
+	prep_status(ctx, "Checking for user columns of enum types");
+
+	snprintf(output_path, sizeof(output_path), "%s/tables_using_enum_types.txt",
+			ctx->home_dir);
+	
+	for (dbnum = 0; dbnum < active_cluster->dbarr.ndbs; dbnum++)
+	{
+		PGresult   *res;
+ 		bool		db_used = false;
+		int			ntups;
+		int			rowno;
+		int			i_nspname, i_relname, i_attname;
+		DbInfo	   *active_db = &active_cluster->dbarr.dbs[dbnum];
+		PGconn	   *conn = connectToServer(ctx, active_db->db_name, whichCluster);
+		
+		/* Find any user-defined tsquery columns */
+		res = executeQueryOrDie(ctx, conn,
+								"SELECT n.nspname, c.relname, a.attname "
+								"FROM	pg_catalog.pg_class c, "
+								"		pg_catalog.pg_namespace n, "
+								"		pg_catalog.pg_attribute a, "
+								"		pg_catalog.pg_type t "
+								"WHERE	c.relkind = 'r' AND "
+								"		c.oid = a.attrelid AND "
+								"		NOT a.attisdropped AND "
+								"		a.atttypid = t.oid AND "
+								"		c.relnamespace = n.oid AND "
+								"		t.typtype = 'e' AND "
+								"		n.nspname != 'pg_catalog' AND "
+								"		n.nspname != 'information_schema'");
+
+		ntups = PQntuples(res);
+		i_nspname = PQfnumber(res, "nspname");
+		i_relname = PQfnumber(res, "relname");
+		i_attname = PQfnumber(res, "attname");
+		for (rowno = 0; rowno < ntups; rowno++)
+		{
+			found = true;
+			if (script == NULL && (script = fopen(output_path, "w")) == NULL)
+					pg_log(ctx, PG_FATAL, "Could not create necessary file:  %s\n", output_path);
+			if (!db_used)
+			{
+				fprintf(script, "Database:  %s\n", active_db->db_name);
+				db_used = true;
+			}
+			fprintf(script, "  %s.%s.%s\n",
+					PQgetvalue(res, rowno, i_nspname),
+					PQgetvalue(res, rowno, i_relname),
+					PQgetvalue(res, rowno, i_attname));
+		}
+
+		PQclear(res);
+
+		PQfinish(conn);
+	}
+
+	if (found)
+	{
+		fclose(script);
+		pg_log(ctx, PG_REPORT, "fatal\n");
+		pg_log(ctx, PG_FATAL,
+				"| Your installation uses enum types.\n"
+				"| These types are not supported for upgrade because\n"
+				"| they contain an internal pg_type.oid that cannot be\n"
+				"| migrated.  You can remove the problem columns and\n"
+				"| restart the migration.  A list of the problem columns\n"
+				"| is in the file:\n"
+				"| \t%s\n\n", output_path);
+	}
+	else
+		check_ok(ctx);
+}
+
+
 /*
  * v8_3_check_for_isn_and_int8_passing_mismatch()
  *
@@ -762,10 +1036,14 @@ v8_3_create_sequence_script(migratorContext *ctx, Cluster whichCluster)
 	}
 	if (found)
 		fclose(script);
-	else	/* mark script as unused */
-		output_path[0] = '\0';
 		
 	check_ok(ctx);
 
-	return output_path;
+	if (found)
+		return output_path;
+	else
+	{
+		pg_free(output_path);
+		return NULL;
+	}
 }
